from math import *
from Ship import Ship
from gutil import *
from Image import Image
import globals
from globals import *

class BaseAI:

    def __init__(self, ship):
        self.moving_to = [0,0]
        self.push = None
        self.push_function = None
        self.push_name = None
        self.push_values = None
        self.ship = ship
        self.turnfudge = 5
        self.fudge = 5
        self.image = Image('blank')
        self.ping = Image("ping")
        self.target = [0,0]
        self.debug = False
        self.mytarget = None
        self.radius = 0
        self.last_angle = 0        
        
    def update(self, dt = 0):
        self.ship.update(dt,[self.mytarget,]+globals.objectList)
        
        self.run_checks()
        
        if self.push:
            self.push = None
            values = (self.push_function, self.push_name, self.push_values)
            return (values)
            
        else:
            return None

    def set_course(self, pos = [0,0], dt=0):
        self.moving_to = pos
        self.keep_course(dt)
        
    def keep_course(self, dt = 0):
        distance = self.ship.dist(pos = self.moving_to)
        self.target[0] = self.moving_to[0] - self.ship.velocity[0]
        self.target[1] = self.moving_to[1] - self.ship.velocity[1]  

        if distance > self.ship.current_velocity**3:
            self.ship.thrust=True
        else:
            self.ship.thrust=False

        if distance < self.radius:
            self.ship.thrust=False

        x1, y1 = self.ship.pos
        x2, y2 = self.target
        
        if y1 == y2:
            beta = 90.0
        else:
            beta = abs(degrees(atan((x1-x2)/(y1-y2))))
        if y1 > y2:
            if x1 > x2:
                quad = 2
            else:
                quad = 1
                beta = 90- beta
        else:
            if x1 > x2:
                quad = 3
                beta = 90 - beta
            else:
                quad = 0
        theta = beta + 90*quad

        if abs(x1-x2) < self.fudge and y1 < y2:
            theta = 0
        if abs(x1-x2) < self.fudge and y1 > y2:
            theta = 180
        if abs(y1-y2) < self.fudge and x1 < x2:
            theta = 90           
        if abs(y1-y2) < self.fudge and x1 > x2:
            theta = 270
            

            
        if theta > self.ship.rotation + self.turnfudge:
            self.ship.rotateDir = 1
        elif theta < self.ship.rotation - self.turnfudge:
            self.ship.rotateDir = -1
        else:
            self.ship.rotateDir = 0
            
        if abs(theta - self.ship.rotation) > 180:
            self.ship.rotateDir *= -1

        if self.ship.percision_override:
            self.ship.rotation = theta
            
        self.last_angle = theta

    def push_ai(self, name, values):
        self.push = True
        self.push_function = 1
        self.push_name = name
        self.push_values = values
        
    def draw(self):
        self.ship.draw()
        
        if self.debug:
            drawLine(start = self.ship.pos, end = self.target, color = (1,0,0,1))
            self.ping.draw(pos = self.target, color=(0,1,1))        
            self.ping.draw(pos = self.ship.pos, color=(1,0,0))

        
    def pop_ai(self):
        self.push = True
        self.push_function = 2


    # Hooks for reacting to things
    
    def run_checks(self):
        #check for player within range
        if self.ship.dist(ship=globals.player.ship) < self.ship.sensor_range and not globals.player.ship.cloaked:
            data = {}
            data['target'] = globals.player.ship
            self.see_threat(data)
    
    def see_threat(self, data=None):
        pass

