import gutil
import pygame
import os
import sys

from random import *
from math import *
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *

from Image import Image
from Text import Text
from Menu import Menu
from TextChunk import TextChunk
from Config import *

import globals
from globals import *

class Intro:

    def __init__(self, levels):
        self.w = globals.DWIDTH
        self.h = globals.DHEIGHT
        self.offset = 50
        screen = (self.w, self.h)
        self.screen = screen
        self.levels = levels
        self.ping = Image('ping')
        self.background = Image('0302062')
        self.box = Image('blank')
        filename = os.path.join('data', 'menu.wav')
        self.sound = pygame.mixer.Sound(filename)
        self.play_sound = False
        self.checklist = (0,0)
        self.selected = (.156, .706, .156, .9)
        self.color = (.125, .565, .156, 1)
        self.font = os.path.join('data', 'StealThisFont.ttf')
        self.title = Text(globals.game_name, fontsize = 90, font = self.font, color = (255,255,255,255))
        
        
        #main screen
        options = []
        options.append((2,"New Game"))
        options.append((3,"Select Level"))        
        options.append((1,"Information"))
        options.append((-1,"Quit Game"))
        self.main = Menu(screen, options, default=1, size = 40,font = self.font)
        

        #levels
        self.level_text = []
        self.make_level_list()
        
        #help
        filename = os.path.join('data','help.text')
        file = open(filename, "r")
        text = file.read()
        self.help = TextChunk(text, fontsize = 25, font = self.font)
        
        self.shown = self.main

    def make_level_list(self):
        options = []
        for i in range(0, globals.config['max_level']+1):
            name_key = "Level"+str(i)+"Name"
            short_key = "Level"+str(i)+"Short"
            if not (globals.config.has_key(name_key) and globals.config.has_key(short_key)):
                self.levels.setLevel(i)
                level = self.levels.getLevel()
                level.initLevel()
                globals.config[short_key] = level.short
                globals.config[name_key] = level.name
                save(globals.config, globals.config_filename) 
                
            short = globals.config[short_key]
            name = globals.config[name_key]
            options.append((i, name))
            text =  TextChunk(short,fontsize = 25,font = self.font)
            self.level_text.append(text)
        self.level_list = Menu(self.screen, options, default=-2, size = 40,font = self.font)

    def update(self):
        self.draw()
        
        main, level = self.checklist
        if main != self.main.position or level != self.level_list.position or self.play_sound:
            self.sound.play()
            self.play_sound = False
            
        self.checklist = (self.main.position, self.level_list.position)

    def draw(self):
        self.background.draw(pos=(0,0), height= self.h, width = self.w)
        
        self.title.draw(pos = ((self.w-self.title.width) / 2, self.h - self.title.height - 10), color = self.color)
        
        x = 100
        y_offset = 150
        self.outline((x,self.main.height + y_offset),(self.main.width + x,y_offset), 12,2)
        self.main.draw((x,self.main.height + y_offset), self.color, self.selected)

        # Draw Levels
        if self.shown == self.level_list:
            x += self.main.width + self.offset
            self.outline((x,self.level_list.height + y_offset),(self.level_list.width + x,y_offset), 12,2)
            self.level_list.draw((x,self.level_list.height + y_offset), self.color, self.selected)
            
            text = self.level_text[self.level_list.position]
            y = y_offset-self.offset
            x = 100

            self.outline((x,y),(text.width + x,y-text.height), 12,2)
            text.draw((x,y), color = self.color)

        # Draw Help
        if self.main.position == 2:
            y = self.help.height + self.offset
            x += self.main.width + self.offset
            self.outline((x,y),(self.help.width + x,y-self.help.height), 12,2)
            self.help.draw((x,y), color = self.color)


    def handleKey(self, type, key):
        state = self.shown.handleKey(type, key)
        
        if self.shown == self.main:
            if type == KEYDOWN and key == K_ESCAPE:
                return (-1, 0)

            if state == 3:
                self.shown = self.level_list
                self.play_sound = True
                return (1, 0)
            else:
                return (state, 0)

        elif self.shown == self.level_list:
            if type == KEYDOWN and (key == K_ESCAPE or key == K_LEFT):
                self.shown = self.main
                self.play_sound = True
                return (1,0)
            elif (state >= 0):
                return (2, state)
                self.shown == self.main
                self.play_sound = True
            else:
                return (1, state)
                
        return (1,0)

    def outline(self, bottom_left, top_right, padding, edge):
        x1, y1 = bottom_left
        x2, y2 = top_right
        
        x1 -= padding + edge
        x2 += padding + edge
        y1 += padding + edge
        y2 -= padding + edge
        
        self.box.draw(pos = (x1, y1), height = y2 - y1 , width = x2 - x1, color = (1,1,1,1))
        
        x1, y1 = bottom_left
        x2, y2 = top_right
        
        x1 -= padding
        x2 += padding
        y1 += padding
        y2 -= padding
        
        self.box.draw(pos = (x1, y1), height = y2 - y1, width = x2 - x1, color = (0,0,0,1))        