
import globals
from Ai import Ai
from Player import Player
from Objects import *
from putil import *
from Splash import Splash
from EnemyShips import *
from PlayerShips import *
import random

class BaseLevel:
    def __init__(self, engine):
        self.engine = engine
        self.introText = ""
        self.not_invicible = True
        self.short = False
    
    def initLevel(self):
        self.engine.player = Player()
        globals.player = self.engine.player

        globals.enemyList = []
        globals.objectList = []
        
        text = self.name + "\n\n" + self.objective + "\n\n" + "(Press Space to begin)"
        
        self.engine.splash = Splash(text)
        self.engine.showSplash = True
    
    def update(self, dt):
        pass
    
    def checkWin(self):
        if globals.enemyList == []:
            return True
        else:
            return False
    
    def checkLose(self):
        if self.engine.player.ship.dead and self.not_invicible:
            return True
        else:
            return False
            
    def getStatus(self):
        return { 'win': self.checkWin(), 'lose': self.checkLose() }




class Level1(BaseLevel):

    def initLevel(self):
        self.name = "Training - Basic Controls"
        self.short = "Learn to fly and destroy the targets."
        self.objective = \
"""Welcome to the alliance recruit.  Lets get your framiliar with the controls of
your craft:

   Up Arrow:   Thrust Forward
   Down Arrow:  Reverse Thrusters
   Left Arrow:  Rotate Left
   Right Arrow:  Rotate Right 
   Left Shift:  Fire

To start with, we will warp you into a sector with 4 target drones.  Take them out."""    


        BaseLevel.initLevel(self)
        
        # init enemies
        points = [
            [300, 550],
            [300, 300],
            [700, 300],
            [700, 550]
             ]
        rotations = [ 180, 90, 0, 270]
        
                 
        for i in range(len(points)):
            data={}
            data['initial']= points[i]
            data['patrol_points']=[points[i],points[i]]
            data['target'] = self.engine.player.ship
            data['passive'] = True
            data['rotation'] = rotations[i]            
            ship = self.ship = BattleDrone()
            globals.enemyList.append(Ai(ship=ship,name="TargetAI",values=data))




class Level2(BaseLevel):

    def initLevel(self):
        self.name = "Training - Moving Targets"
        self.short = "Destroy the moving targets."
        self.objective = \
"""Well done!  You might just pass out of training after all.
Lets try something a bit harder this time.  See if you can
manage some moving targets.

We've deployed four mobile target drones.  Destroy them
to finish this task."""    


        BaseLevel.initLevel(self)
        
        # init enemies
        points = [
            [300, 550],
            [300, 300],
            [700, 300],
            [700, 550]
             ]
             
        rotations = [ 180, 90, 0, 270]
                 
        for i in range(len(points)):
            data={}
            data['initial']= points[i]
            data['patrol_points']=points[i:len(points)]+points[0:i] # rotate the list about i
            data['target'] = self.engine.player.ship
            data['passive'] = True
            data['rotation'] = rotations[i]
            ship = self.ship = BattleDrone()
            globals.enemyList.append(Ai(ship=ship,name="Patrol",values=data))



class Level3(BaseLevel):
    def initLevel(self):
        self.name = "Training - Inertialess Ship"
        self.short = "Destroy the targets while flying without inertia."
        self.objective = \
"""Ok, it looks like the you've got the basics down.  Not
every ship has Inertial Dampers though.  Let's try the
same exercise in a smaller, faster ship.

Which ship you use in the field will depend on the mission
specs.  The controls are tricky on this one but in the
hands of a good pilot it can be a far greater threat to
our enemies.

Four more target drones have been deployed.  Report
back when you have taken them out."""

        BaseLevel.initLevel(self)
        
        # init enemies
        points = [
            [300, 550],
            [300, 300],
            [700, 300],
            [700, 550]
             ]
             
        rotations = [ 180, 90, 0, 270]
                 
        for i in range(len(points)):
            data={}
            data['initial']= points[i]
            data['patrol_points']=points[i:len(points)]+points[0:i] # rotate the list about i
            data['target'] = self.engine.player.ship
            data['passive'] = True
            data['rotation'] = rotations[i]            
            ship = self.ship = BattleDrone()
            globals.enemyList.append(Ai(ship=ship,name="Patrol",values=data))
            
        self.engine.player.ship = SmallHero()
        #globals.player


class Level4(BaseLevel):
    def initLevel(self):
        self.name = "Training - Cloaking"
        self.short = "Bypass a minefield."
        self.objective = \
"""Well done.  Lets move on to the more advanced features of your ship.

The Canopy outnumbers us.  Their weapons are more powerful, and they out range our best ships.
We rely on speed, and superior sensors to survive.  But our biggest advantage is the cloaking
device we equip all of our ships with.

The cloak is engaged and disengaged with the 'c' key.
The tab key will give you detailed info on your enemies. 'q' to cycles back.

Traverse the mine field, to get to the waypoint.  Be careful, these laser mines are live, and
will shoot if they detect you."""

        BaseLevel.initLevel(self)
        
        points = []
        for i in range(3):
            for j in range(8):
                xr = random.randrange(-15,15)
                yr = random.randrange(-25,25)
                points.append([500+i*50+xr, j/8.0*globals.DWIDTH+yr])
                
        for i in range(len(points)):
            data={}
            data['initial']= points[i]

            data['target'] = self.engine.player.ship
            ship = self.ship = LaserMine()
            globals.enemyList.append(Ai(ship=ship,name="MineAI",values=data))

        # init objects        
        globals.objectList.append(WayPoint((800,300)))


    def checkWin(self):
        if circleCollide(self.engine.player.ship, globals.objectList[0]):
            return True
        else:
            return False


class Level5(BaseLevel):

    def initLevel(self):
        self.name = "All Alone"
        self.short = "Destroy the enemy ship."
        self.objective = \
"""The resources of The Canopy Group are far more vast than our
own.  For this reason we hit them when we can.  Your first
live assignment is to take out a lone Canopy scout ship in
this sector.

Good Luck!"""    

        BaseLevel.initLevel(self)
        
        data={}
        data['initial']= [600,600]
        data['patrol_points']= [[700,700], [600,600]]
        data['target'] = self.engine.player.ship
        globals.enemyList.append(Ai(name="Patrol", values=data))

        
        self.engine.player.ship = SmallHero()

            
            
        
class Level6(BaseLevel):
    def initLevel(self):
        self.name = "Medical Delivery"
        self.short = "Deliver the package past the blockade."
        self.objective = \
"""In order to force the peaceful citizens of this sector to
join them, Canopy has blockaded their planet.  No trade in
or out is allowed.  Even when a plague broke out they refused
access to aid transports.

We need you to smuggle a shipment of medicine past the blockade
to the planet.  Once you reach the surface they can hide you
and your ship until your next mission.  Remember, lives are
depending on you."""

        BaseLevel.initLevel(self)
        
        # init enemies
        points = [
            [550, 600],
            [550, 400],
            [750, 400],
            [750, 600]
             ]

        rotations = [ 180, 90, 0, 270]
         
        for i in range(3):
            data={}
            data['initial']= points[i]
            data['patrol_points']=points[i:len(points)]+points[0:i] # rotate the list about i
            data['target'] = self.engine.player.ship
            data['rotation'] = rotations[i]
            globals.enemyList.append(Ai(name="Patrol", values=data))

        
        # init objects        
        globals.objectList.append(SmallMoon((650,500)))
        
    def checkWin(self):
        if circleCollide(self.engine.player.ship, globals.objectList[0]):
            return True
        else:
            return False

class Level7(BaseLevel):
    def initLevel(self):
        self.name = "Diplomatic Mission"
        self.short = "Escort the senator."
        self.objective = \
"""After the success of your last mission it was decided to 
promote you!  Don't get cocky though Lietenant, most pilots
only last half a dozen or so missions.  Keep sharp out there

Remember that planet you helped last time?  Well Canopy
has pulled out but left a nasty supprise for them. Laser
mines all around.  We need you to make another run through
them.  This time your cargo is a slightly unwilling Senator.

We need more people on our side and showing them the harm
Canopy is doing is the best way.
"""
        BaseLevel.initLevel(self)
        
        # init enemies
        points = [
            [550, 600],
            [550, 400],
            [750, 400],
            [750, 600]
             ]

        rotations = [ 180, 90, 0, 270]
         
        for i in range(len(points)):
            data={}
            data['initial']= points[i]
            #data['patrol_points']=points[i:len(points)]+points[0:i] # rotate the list about i
            data['target'] = self.engine.player.ship
            #data['rotation'] = rotations[i]
            ship = self.ship = LaserMine()
            globals.enemyList.append(Ai(ship=ship,name="MineAI",values=data))

        
        # init objects        
        globals.objectList.append(SmallMoon((650,500)))
        
    def checkWin(self):
        if circleCollide(self.engine.player.ship, globals.objectList[0]):
            return True
        else:
            return False
            
            
class Level8(BaseLevel):
    def initLevel(self):
        self.name = "Clear A Minefield"
        self.short = "Destroy all the mines"
        self.objective = \
"""The minefield has since grown.  We need to get more supplies
through and we just don't don't have enough stealth ships.
Since you're in the area that duty falls on you.

Clear the minefield.  And for your sake, don't die doing it."""

        BaseLevel.initLevel(self)
        
        points = []
        for i in range(4):
            for j in range(4):
                xr = random.randrange(-30,30)
                yr = random.randrange(-50,50)
                points.append([100+i*140+xr, j/8.0*globals.DWIDTH+yr+100])
                
        for i in range(len(points)):
            data={}
            data['initial']= points[i]

            data['target'] = self.engine.player.ship
            ship = self.ship = LaserMine()
            globals.enemyList.append(Ai(ship=ship,name="MineAI",values=data))
            self.engine.player.ship.pos=[700, 550]

        # init objects        
        globals.objectList.append(SmallMoon((650,500)))


class Level9(BaseLevel):
    def initLevel(self):
        self.name = "Assassin Strike"
        self.short = "Pick off a single ship in a convoy."
        self.objective = \
"""Listen up.  This mission is going to be a lot trickier than
the ones we've given you so far.  And it far more vital to
the rebellion as well.  It is a chance to hurt Canopy where
it hurts.  In upper management.

The VP of Advanced Munitions, that's a fancy way of saying
nerve gas and nukes, is going to be travelling through your
sector.  Needless to say, we want him eliminated.

The problem is that he isn't stupid, but he is evil.  This
VP will be traveling as part of a civillian convoy.  You
absolutely can not destroy any civillians!  Even though
they may fire on you, your target is the VP and the VP
alone.  His ship will be the smaller one.

Do not fail this mission.  It is of vital importance.
"""

        BaseLevel.initLevel(self)
        
        data={}
        data['patrol_points']= [
            [globals.DWIDTH-50,globals.DHEIGHT-100], 
            [20,globals.DHEIGHT-100],
            [(globals.DWIDTH-20)/3,(globals.DHEIGHT-20)*2/3],
            [(globals.DWIDTH-20)*3/4,(globals.DHEIGHT-20)*1/3],
            ]
        data['target'] = self.engine.player.ship
        ship = BigAss()
        globals.enemyList.append(Ai(name="Patrol", values=data, ship = ship))
        
        data['friendly'] = globals.enemyList[0].ship
        data['radius'] = 100
        ship = BigAss()
        globals.enemyList.append(Ai(name="Escort", values=data, ship = ship))
               
        data['friendly'] = globals.enemyList[1].ship
        self.target_ship = BattleDrone()
        self.todestroy = Ai(name="Escort", values=data, ship = self.target_ship)
        globals.enemyList.append(self.todestroy)

        ship = BigAss()
        data['friendly'] = globals.enemyList[2].ship
        data['radius'] = 100
        globals.enemyList.append(Ai(name="Escort", values=data, ship = ship))

    def checkWin(self):
        flag = False
        
        for enemy in globals.enemyList:
            if enemy.ship == self.target_ship:
                flag = True
        
        if not flag:
            return True
        else:
            return False
            
            
    def checkLose(self):
        if len(globals.enemyList) < 4:
            return True
        else:
            return BaseLevel.checkLose(self)
            
            
class Level10(BaseLevel):
    def initLevel(self):
        self.name = "Bag The Big One"
        self.short = " Can you take out the Battleship?"
        self.objective = \
"""Wow.  I can't believe you're not dead! Er.. I mean, good
work!  I had faith in you all along!  And to reward you for
your wonderful work we have an easy assignment for you.

A lone ship, just one, needs to be destroyed.  It should be
a cakewalk for you after all you've been through."""

        BaseLevel.initLevel(self)

        
        data={}
        data['initial']= [600,600]
        data['patrol_points']= [
            [globals.DWIDTH-50,globals.DHEIGHT-100], 
            [(globals.DWIDTH-20)*3/4,(globals.DHEIGHT-20)*1/3],
            [20,globals.DHEIGHT-100],
            [(globals.DWIDTH-20)/4,(globals.DHEIGHT-20)*2/3],
            ]
        data['target'] = self.engine.player.ship
        ship = BattleCruiser()
        globals.enemyList.append(Ai(name="Patrol", values=data, ship=ship))
        
        self.engine.player.ship = SmallHero()