import pygame
from pygame.locals import *
from Text import Text
from Image import Image

#Makes a list of options
class Menu:
    
    # screen = a tuple describing the width and height of the screen
    # options = a list of tuples in the format (payload, text) where payload will be returned when text is selected
    # background = a background image with a draw function
    # default = the default payload
    # size = the font size
    def __init__(self, screen, options, default = None, size = 14, background = None, font = None):
        self.w = screen[0]
        self.h = screen[1]
        
        self.default = default
        self.background = None
        if background != None:
            self.background = background
            
        # Sets up the list of items
        self.options = []
        for item in options:
            payload, string = item
            self.options.append((payload, Text(string, color = (255,255,255,255), fontsize = size, font = font)))
            
        # gets the max width and total height of the menu
        self.height = 0
        self.width = 0
        for item in self.options:
            discard, text = item
            if self.width < text.width:
                self.width = text.width
            self.height += text.height

        # Tracks where in the list we are
        self.position = 0
        
        
    #draws the list to the screen on a vertical offset
    def draw(self, pos=(0,0), color=(1,1,1,1), selected_color=(.8,.8,.8,.9), centered = 0):
        x, y = pos
        x_offset = 0
        
        if self.background:
            self.background.draw(width = self.w, height = self.h)
        
        for i in range(len(self.options)):
            chuck, item = self.options[i]
            y -= item.height
            menu_color = color
            if i == self.position:
                menu_color = selected_color
            if centered:
                x_offset = (self.width - item.width) / 2
            item.draw(pos = (x+ x_offset, y), color = menu_color)
        
    def handleKey(self, type, key):
        if type == KEYDOWN:
            if key == K_UP:
                self.position -= 1
                if self.position < 0:
                    self.position = len(self.options) - 1
            elif key == K_DOWN:
                self.position += 1
                if self.position > len(self.options) - 1:
                    self.position = 0
                    
            elif key == K_SPACE or key == K_RETURN or key == K_RIGHT:
                x, y = self.options[self.position]
                return x
                
        return self.default