
from pygame.locals import *

from Image import Image
from Text import Text


from Ship import *
from PlayerShips import *
from Weapon import Weapon

import globals
from globals import *

class Player:
    def __init__(self):
        self.ship = HeroShip()
        self.boundsFudge = 10 # percent
        self.hudBar = Image('blank')
        self.sensor = Image('sensor')
        self.selector = Image('selector')
        self.target = None
    
    def setShip(self, newShip):
        self.ship = newShip
        
    def update(self,dt):
        if not self.target is None:
            #if self.target.dead:
                
            if self.target > len(globals.enemyList) -1:
                self.target = None
    
        self.ship.update(dt,[enemy.ship for enemy in globals.enemyList]+globals.objectList)
        
        
        # Handle ship out of bounds
        
        shipSize = self.ship.width
                
        # Using Arrays As Control Structure Considered Harmful?
        boundsParams = (
            ( lambda a,b : a < b, 0,0 - BOUNDS_GRACE, 90, 0 - shipSize, 1.0 ),
            ( lambda a,b : a > b, 0,DWIDTH + BOUNDS_GRACE - shipSize, 270, DWIDTH, -1.0 ),
            ( lambda a,b : a > b, 1,DHEIGHT + BOUNDS_GRACE - shipSize, 180, DHEIGHT, -1.0 ),
            ( lambda a,b : a < b, 1,0 - BOUNDS_GRACE, 0, 0 - shipSize, 1.0 )
           )
        # array format is:
        # comparision function, x or y pos, edge to compare against, new rotation, new position, new direction
        
          
        for p in boundsParams:
            if p[0](self.ship.pos[p[1]],p[2]):
                if shipSize > BOUNDS_GRACE:
                    print "Bounds checking will probably break on big ships."

                self.ship.velocity = [0,0]
                self.ship.velocity[p[1]] = self.ship.maxVel * 0.1 * p[5]
                self.ship.rotation = p[3]
                self.ship.thrust = 0
                self.ship.rotateDir = 0
                self.ship.pos[p[1]] = p[4]
                
            
            
        
    def draw(self):
        if not self.target is None:
            enemy = globals.enemyList[self.target]
            size = enemy.ship.sensor_range * 2
            pos = enemy.ship.pos[0] - size/2.0, enemy.ship.pos[1] - size/2.0
            self.sensor.draw(pos=pos , width = size, height = size, color =(0.1,0.1,0.6,0.2))

        self.ship.draw()
        
        self.hudBar.draw(pos=(10,10), width=150, height = 10, color=[1.0,1.0,1.0,1.0])
        self.hudBar.draw(pos=(10,10), width=150.0*self.ship.cloakEnergy/self.ship.cloakMaxEnergy, height = 10, color=[0.3,0.3,1.0,1.0])       

        self.hudBar.draw(pos=(10,30), width=150, height = 10, color=[1.0,1.0,1.0,1.0])
        if self.ship.health > 0:
            self.hudBar.draw(pos=(10,30), width=150.0*self.ship.health/self.ship.maxHealth, height = 10, color=[0.3,1.0,0.3,1.0])
            
    def draw2(self):
        if not self.target is None:
            enemy = globals.enemyList[self.target]
            size = enemy.ship.width * 1.6
            pos = enemy.ship.pos[0] - size/2.0, enemy.ship.pos[1] - size/2.0
            self.selector.draw(pos=pos , width = size, height = size, color=[1,1,1,0.5])
            
            self.hudBar.draw(pos =[pos[0]+size + 4, pos[1]+size - 8], width=50,height =6 , color=[1.0,1.0,1.0,0.7])
            if enemy.ship.health > 0:
                self.hudBar.draw(pos =[pos[0]+size +4, pos[1]+size -8], width=50.0*enemy.ship.health/enemy.ship.maxHealth,height =6 , color=[0,0.7,0,0.4])
 
    def handleKeyboardEvent(self,event):
        jumpTable = {
            KEYDOWN : {
                K_UP: self.setThrust,
                K_DOWN: self.reverseThrust,
                K_LEFT: self.rotGen(-1),
                K_RIGHT: self.rotGen(1),
                K_LSHIFT: self.setFiring(True),
                K_c: self.toggleCloak,
                K_TAB: self.nextTarget,
                K_q: self.prevTarget
                },
            KEYUP : {
                K_UP: self.thrustOff,
                K_DOWN: self.reverseThrustOff,
                K_LEFT: self.stopGen(-1),
                K_RIGHT: self.stopGen(1),
                K_LSHIFT: self.setFiring(False),
                K_i: self.toggleInertialDamp
                }
            }

        if jumpTable.has_key(event.type):
            if jumpTable[event.type].has_key(event.key):
                jumpTable[event.type][event.key]()
    
    
    ####### input jump functions #######

    def nextTarget(self):
        if self.target is None:
            self.target = 0
        else:
            self.target += 1

    def prevTarget(self):
        if self.target is None:
            self.target = len(globals.enemyList) -1
        elif self.target == 0:
            self.target = None
        else:
            self.target -= 1


    def rotGen(self, dir):
        def rotateShip():
            self.ship.rotateDir = dir
            
        return rotateShip
        
    def stopGen(self, dir):
        def stopRotate():
            if self.ship.rotateDir == dir:
                self.ship.rotateDir = 0
        
        return stopRotate


    def setFiring(self, state):
        """Used to just flip the state, now does this so that
           if you level up firing, the right thing happens."""
        def _():
            self.ship.firing = state
        
        return _ # _() is as close as python gets to an annon function

    def toggleCloak(self):
        self.ship.toggleCloak()
        #self.ship.cloaked = not self.ship.cloaked
        self.ship.cloak_channel.play(self.ship.cloak_sound)
        self.ship.cloak_channel.set_volume(0.6)
        
    def setThrust(self):
        self.ship.thrust = True
        self.ship.reverseThrust = False
        
    def thrustOff(self):
        self.ship.thrust = False

    def reverseThrust(self):
        self.ship.thrust = False
        self.ship.reverseThrust = True
        
    def reverseThrustOff(self):
        self.ship.reverseThrust = False

    def toggleInertialDamp(self):
        self.ship.toggleFriction()
        
        