
from Image import Image
from Weapon import *
from ParticleFountain import *

import pygame
import os
import globals
from math import *

class Ship:
    def __init__(self, 
                    shipImage='scout',
                    velocity=[0,0], 
                    pos=[0,0], 
                    rotation = 0, 
                    rotateVel = 2.0, 
                    accel = 0.02, 
                    maxVel = 1.5, 
                    cloakEnergy = 100, 
                    cloakCharge= 1.0,
                    cloakCost= 0.5,
                    health=100,
                    friction=0,
                    percision_override=False,
                    sensor_range=300,
                    size=32,
                    weapon=Weapon,
                    mass=16):
    
        if type(shipImage) == type(""):
            self.image = Image(shipImage)
        else:
            self.image = shipImage
            
        self.width = size
        self.height = size
        self.radius = size * 0.4 # hit area
            
        self.velocity = velocity[:]
        self.current_velocity = sqrt(self.velocity[0]**2 + self.velocity[1]**2)
        self.friction = friction
        self.pos = pos[:]
        self.rotation = rotation
        
        self.sensor_range = sensor_range
        self.rotateDir = 0
        self.thrust = False
        self.reverseThrust = False
        
        self.rotateVel = rotateVel
        self.accel = accel
        self.maxVel = maxVel
        
        self.cloaked = False
        
        self.cloakCost = cloakCost
        
        self.weapon = weapon()
        self.exhaust = self.initEngines()
        
        self.cloakMaxEnergy = cloakEnergy
        self.cloakEnergy = cloakEnergy
        self.cloakCharge = cloakCharge
        self.health = health
        self.maxHealth = self.health
        self.dead = False
        self.dying = False
        self.percision_override=percision_override
        self.firing = False
        self.mass = mass
        self.explosion = ParticleFountain(image ='ping', thrustVel = 1, size = 20, ttl = 15, rate = 1, particletype = 'RandomParticle')
        self.dieTime = 25 # explode time
        self.dieCount = 0
        
        #sounds
        filename = os.path.join('data', 'explosion.wav')
        self.explosion_sound = pygame.mixer.Sound(filename)
        filename = os.path.join('data', 'cloak.wav')
        self.cloak_sound = pygame.mixer.Sound(filename)
        self.cloak_channel = pygame.mixer.Channel(1)

    def update(self,dt, targetList):
        if self.dying:
            self.cloaked = False
            self.thrust = False
            self.firing = False
            self.thrust = False
            self.reverseThrust = False
    
        if self.rotateDir != 0 and not self.dying:
            self.rotation += self.rotateDir * self.rotateVel * dt
            if self.rotation > 360:
                self.rotation -= 360
            elif self.rotation < 0:
                self.rotation += 360
                        
        if (self.thrust or self.reverseThrust) and not self.dying:
            dxv = sin(radians(self.rotation)) * self.accel * dt
            dyv = cos(radians(self.rotation)) * self.accel * dt

            if self.thrust:
                self.velocity[0] += dxv
                self.velocity[1] += dyv
            elif self.reverseThrust:
                self.velocity[0] -= dxv
                self.velocity[1] -= dyv
                        
            linVelocity = (self.velocity[0]**2 + self.velocity[1]**2)**0.5
            if linVelocity > self.maxVel:
                self.velocity[0] = self.velocity[0]/linVelocity * self.maxVel
                self.velocity[1] = self.velocity[1]/linVelocity * self.maxVel
        else:
            if self.friction:
                self.velocity[0] *= 1-(self.friction * dt)
                self.velocity[1] *= 1-(self.friction * dt)

        # cache old pos incase movement puts us inside another object
        oldPos = self.pos

        self.pos[0] += self.velocity[0]
        self.pos[1] += self.velocity[1]

        # test for collisions
        for thing in [enemy.ship for enemy in globals.enemyList] + globals.objectList + [globals.player.ship,]:
            if thing and thing != self:
                if circleCollide(self, thing):
                    self.pos = oldPos
                    newVel = circleRespond(self,thing)
                    thing.velocity = circleRespond(thing,self)
                    self.velocity = newVel
                    # collisions break cloak
                    self.decloak()
                    try:
                        thing.decloak()
                    except AttributeError:
                        pass # thing doesn't have a decloak method.  Oh well.
                    break

        self.updateExhaust(dt)
        
        if not self.dying:
                    
            if self.cloaked:
                self.cloakEnergy -= self.cloakCost * dt
                
                if self.cloakEnergy < 0:
                    self.cloakEnergy = 0
                    self.cloaked = False
                    self.cloak_channel.play(self.cloak_sound)
                    self.cloak_channel.set_volume(0.6)
            
            elif self.cloakEnergy < self.cloakMaxEnergy:
                self.cloakEnergy += self.cloakCharge * dt
                if self.cloakEnergy > self.cloakMaxEnergy:
                    self.cloakEnergy = self.cloakMaxEnergy

        else:
            self.dieCount += dt
            moreExplosion = True
            if 1.0 * self.dieCount / self.dieTime > 0.9:
                moreExplosion = False
            self.explosion.update(dt, self.pos, 0, moreExplosion)
            if self.dieCount >= self.dieTime:
                self.dead = True


        self.weapon.firing = self.firing
        point = [0,0]
        point[0] = self.pos[0] + sin(radians(self.rotation)) * self.radius #self.width/2.0 * 0.9
        point[1] = self.pos[1] + cos(radians(self.rotation)) * self.radius #self.width/2.0 * 0.9
        self.weapon.pos = point
        self.weapon.rotation = self.rotation                            
        self.weapon.update(dt, targetList)

        
        self.current_velocity = sqrt(self.velocity[0]**2 + self.velocity[1]**2)     

    def die(self):
        self.cloaked = False
        self.dying = True
        self.explosion_sound.play()
        self.explosion_sound.set_volume(0.6)

    def draw(self):
        self.weapon.draw()
        for e in self.exhaust:
            e.draw()
            
        
        if self.cloaked:
            color=[1.0,1.0,1.0,0.3] 
        elif self.dying and 1.0 * self.dieCount / self.dieTime > 0.9:
            color=[1.0,1.0,1.0,1.0 - 10.0 * self.dieCount / self.dieTime]
        else:
            color=[1.0,1.0,1.0,1.0] 

        self.image.draw(pos=(self.pos[0] - self.width/2.0,self.pos[1] - self.height/2.0) ,rotation=self.rotation, rotationCenter=(self.width/2.0,self.height/2.0), width=self.width, height=self.height,color=color)
     
        self.explosion.draw()

    def dist(self, pos=None, ship = None):
        if ship:
            x2, y2 = ship.pos
        elif pos:
            x2,y2 = pos
        else:
            return -1
            
        x1,y1 = self.pos
        return sqrt((x1-x2)**2+(y1-y2)**2)
        
    def decloak(self):
        if self.cloaked:
            self.toggleCloak()

    
    def toggleCloak(self):
        self.cloaked = not self.cloaked
    
    def toggleFriction(self):
        if self.friction == 0:
            self.friction = 0.025
        else:
            self.friction = 0
            
    def updateExhaust(self, dt):
        point = [0,0]
        point[0] = self.pos[0] - sin(radians(self.rotation)) * self.radius
        point[1] = self.pos[1] - cos(radians(self.rotation)) * self.radius
        for e in self.exhaust:
            e.update(dt, point, self.rotation, self.thrust)


    def initEngines(self):
        return [ParticleFountain()]