
from Image import Image

from ParticleFountain import ParticleFountain
import os
import pygame

from random import *
from math import *

from putil import *

class Weapon:
    def __init__(self):
        self.range = 100.0
        self.width = 5.0
        self.damage = 1.0
        self.charge = 100.0
        self.drainRate = 1.0
        
        self.image = Image('blank')
        self.rotation = 0
        self.firing = False
        self.pos = [0,0]
        self.rotation = 0
        self.terminationPoint = [0,self.range]
        #self.boundingRect
        self.sampleRate = 10
        self.radius = self.range
        self.hitRange = 1.0
        self.sparks = ParticleFountain(image ='ping', thrustVel = 5, size = 10, ttl = 10, rate = 1, particletype = 'SparkParticle')

        filename = os.path.join('data', 'laser.wav')
        self.shoot_sound = pygame.mixer.Sound(filename)
        self.channel = pygame.mixer.Channel(2)
        self.channel.set_volume(0.7)

    def draw(self):        
        if self.firing:
            color = [uniform(0,1),uniform(0,1),uniform(0,1),1.0]
            width = self.width + uniform(0,self.width/2.0) - self.width/2.0
            
            self.image.draw(pos=(self.pos[0]-width/2.0,self.pos[1]), rotation=self.rotation, rotationCenter = (width/2.0, 0),  height = self.range*self.hitRange, width = width, color=color)
      
        self.sparks.draw()
    
    def update(self,dt,targetList):
        self.hit = False
        self.hitRange = 1.0
        
        closest = 1.0
        hitting = None
        
        if self.firing:
            if not self.channel.get_busy():
                self.channel.play(self.shoot_sound)
        
            for target in targetList:
                if target:
                    hit, range = self.hitTarget(target)
                    if hit:
                        if range <= closest:
                            closest = range
                            hitting = target
        
        if hitting:
            self.hit = True
            self.hitRange = closest
            
            hitting.health -= self.damage * dt
            if hitting.health < 0:
                hitting.die()
                
            self.sparks.update(dt, self.getPoint(self.range*self.hitRange), 0, True)
        else:
            self.sparks.update(dt, [0,0], 0, False)

    def getPoint(self,dist):
        point = [0,0]
        point[0] = self.pos[0] + sin(radians(self.rotation)) * dist
        point[1] = self.pos[1] + cos(radians(self.rotation)) * dist
        
        return point
    
    def hitTarget(self, target):        
        hit = False
        hitRange = 1.0
        if circleCollide(HitPoint(self.pos,self.range),target):
            for i in range(self.sampleRate+1):
                point = self.getPoint(1.0*self.range*i/self.sampleRate)
                if circleCollide(HitPoint(point,6),target):
                    hit = True
                    hitRange = 1.0*i/self.sampleRate
                    break
            
        return hit, hitRange

class EnemyWeapon(Weapon):
    def __init__(self):
        Weapon.__init__(self)
        self.range = 120.0
        self.width = 8.0
        self.sampleRate = 12
        self.radius = self.range


class SuperWeapon(Weapon):
    def __init__(self):
        Weapon.__init__(self)
        self.range = 260.0
        self.width = 12.0
        self.sampleRate = 14
        self.damage = 1.2
        self.radius = self.range

